/**
  *  \file server/request.cc
  */

#include <cstring>
#include <unistd.h>
#include "server/request.h"

Request::Request()
    : length(0),
      read_ptr(0)
{ }

/** Read a single line from buffer. The result will be zero-terminated and modifyable.
    \return line; null if no full line in buffer */
char*
Request::readLine()
{
    size_t end = read_ptr;
    while (end < length && buffer[end] != '\n')
        ++end;

    if (end < length) {
        /* Found a newline within the current buffer */
        char* result = &buffer[read_ptr];
        buffer[end] = 0;
        if (end > read_ptr && buffer[end-1] == '\r')
            buffer[end-1] = 0;
        read_ptr = end+1;
        return result;
    } else {
        /* No newline found, which means we have a partial buffer */
        return 0;
    }
}

/** Continue reading. */
Request::State
Request::readMore(int fd)
{
    /* Ensure maximum request size */
    if (length >= MAX_LENGTH) {
        return sTooLong;
    }

    /* Read data from socket */
    int n = read(fd, &buffer[length], MAX_LENGTH - length);
    if (n < 0) {
        return sError;
    }
    if (n == 0) {
        return sEOF;
    }
    length += n;
    return sSuccess;
}

/** Remove read data from buffer to make room.
    \retval true We could make some room
    \retval false We could not make room */
bool
Request::shift()
{
    if (read_ptr == 0) {
        return false;
    } else {
        std::memmove(&buffer[0], &buffer[read_ptr], length - read_ptr);
        length -= read_ptr;
        read_ptr = 0;
        return true;
    }
}

/** Clear request. Same as default-construction. */
void
Request::clear()
{
    length   = 0;
    read_ptr = 0;
}
